<?php
/**
 * Created by PhpStorm.
 * User: eugeniofonseca
 */

namespace Service\Integration;


use Util\Messages;
use Zeedhi\Framework\DTO\Request;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Util\Utilities;

class SMSoliservIntegration {

    const ACCEPTED_PROPOSAL = "/IntegracaoPm/aceitarProposta";
    const CANCEL_PROPOSAL = "/IntegracaoPm/cancelarProposta";
    const LINK_DOCUMENT_PROPOSAL = "/IntegracaoPm/gerarDocumentosProposta";

    private static function getDataRequestDataSet($dataSet) {
        $dataSet = array(
            "requestType" => Request::TYPE_DATA_SET,
            "dataset" => $dataSet
        );

        return $dataSet;
    }

    private  static function getDataRequestRow($row) {
        $dataRow = array(
            "requestType" => Request::TYPE_ROW,
            "row" => $row
        );

        return $dataRow;
    }

    public static function integrationSMAccept($nrsoliserv, $organizationId)
    {
        try {
            $smRow = self::getRowIntegration($nrsoliserv, $organizationId);
            if ($smRow['id'] && self::validateAccept($nrsoliserv)) {
                $smDataSet = self::getDataRequestDataSet(array($smRow));
                $token = LibTokenIntegration::requestToken($smRow['urlSm']);
                self::requestSMRoute(self::ACCEPTED_PROPOSAL, $token, $smDataSet, $smRow['urlSm']);
                
                
            }
        } catch (\Exception $e) {
            throw new \Exception(Messages::ERR_INTEGRATION_SM . $e->getMessage(), $e->getCode());
        }
    }

    public static function linkDownloadBudgetSm($nrsoliserv, $organizationId)
    {
        $linkDownloadBudgetSm = array();
        try {
            $smRow = self::getRowIntegration($nrsoliserv, $organizationId);
            if ($smRow['id']) {
                $smDataSet = self::getDataRequestDataSet(array($smRow));
                $token = LibTokenIntegration::requestToken($smRow['urlSm']);
                $linkDownloadBudgetSm = self::requestSMRoute(self::LINK_DOCUMENT_PROPOSAL, $token, $smDataSet, $smRow['urlSm']);
                if(isset($linkDownloadBudgetSm['documentos']) && isset($linkDownloadBudgetSm['documentos']['links']) ){
                    $linkDownloadBudgetSm = $linkDownloadBudgetSm['documentos']['links'];
                }
                if(is_array($linkDownloadBudgetSm)){
                    $linkDownloadBudgetSm = $linkDownloadBudgetSm[0];
                }
            }
        } catch (\Exception $e) {
            throw new \Exception(Messages::ERR_INTEGRATION_SM . $e->getMessage(), $e->getCode());
        }
        return $linkDownloadBudgetSm;
    }

    public static function integrationSMCancel($nrsoliserv, $organizationId)
    {
        try {
            $smRow = self::getRowIntegration($nrsoliserv, $organizationId);
            if ($smRow['id']) {
                $smDataSet = self::getDataRequestDataSet(array($smRow));
                $token = LibTokenIntegration::requestToken($smRow['urlSm']);
                self::requestSMRoute(self::CANCEL_PROPOSAL, $token, $smDataSet, $smRow['urlSm']);
            }
        } catch (\Exception $e) {
            throw new \Exception(Messages::ERR_INTEGRATION_SM . $e->getMessage(), $e->getCode());
        }
    }

    public static function requestSMRoute($route,$token, $data = array(),$url){

        $data = json_encode($data);
        $url = $url.$route;

        $header = array();
        $header[] = 'OAuth-Token: '.$token;
        $header[] = 'OAuth-SecretId: '.LibTokenIntegration::CLIENT_SECRET;
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: '.strlen($data);
        $header[] = 'Expect:';

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_TIMEOUT, 86400);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);
        curl_close($curl_session);
        $result = json_decode($result, true);
        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else if (!empty($result['messages'])) {
            throw new \Exception($result['messages'][0]['message']);
        }  else {
            $result = isset($result['dataset']) ? $result['dataset'] : null;
        }

        return $result;
    }
    public static function validateAccept($nrsoliserv)
    {
        $validate = true;
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        if($budgetRepository->validateAcceptProposalSoliservAlreadyAccepted($nrsoliserv)){
            $validate = false;
        }

        return $validate;
    }

    public static function getRowIntegration($nrsoliserv,$organizationId)
    {
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        $baseUri = Utilities::getBaseUri();
        /** @var Entities\Allocation $reporter */
        $urlSm = $budgetRepository->getUrlIntegrationSM($organizationId,$baseUri);
        if($urlSm != $baseUri){
            $proposalValues = $budgetRepository->getProposalValuesSoliserv($nrsoliserv);
            $cdOperCurrent = \Util\Utilities::getEnvironment()->getCurrentUserId();
            $smRow = array(
                "urlSm" => $urlSm,
                "requester" => array(
                    "userId" => $cdOperCurrent,
                    "organizationId" => $organizationId,
                ),
                "id" => $proposalValues['EXTERNAL_ID'],

            );
        }else{
            $smRow = array(
                "id" => null,
            );
        }
        return $smRow;
    }

}
